use welds::WeldsModel;
use welds::migrations::{create_table, types::Type, MigrationStep, TableState};

#[derive(Debug, WeldsModel)]
#[welds(schema = "", table = "objects")]
pub struct Object {
    #[welds(primary_key)]
    pub id: i32,
    pub name: String,
    pub flags: String,

    pub password: Option<String>,
    pub description: Option<String>,
    pub health: Option<i32>,
    pub max_health: Option<i32>,
    pub magic: Option<i32>,
    pub max_magic: Option<i32>,
}

pub fn setup_Objects(_state: &TableState) -> welds::errors::Result<MigrationStep> {
    let m = create_table("objects")
        .id(|c| c("id", Type::IntBig))
        .column(|c| c("name", Type::String).create_unique_index())
        .column(|c| c("flags", Type::String).create_index())

        .column(|c| c("password", Type::String).is_null())
        .column(|c| c("description", Type::String).is_null())
        .column(|c| c("health", Type::Int).is_null())
        .column(|c| c("max_health", Type::Int).is_null())
        .column(|c| c("magic", Type::Int).is_null())
        .column(|c| c("max_magic", Type::Int).is_null());
    Ok(MigrationStep::new("create_table_objects", m))
}

impl Object {
    pub fn has_flag(&self, flag: &str) -> bool {
        self.flags.contains(flag)
    }
    pub fn add_flag(&mut self, flag: &str) {
        if self.has_flag(flag) {
            return;
        }
        self.flags.push_str(flag);
    }
    pub fn remove_flag(&mut self, flag: &str) {
        if !self.has_flag(flag) {
            return;
        }
        self.flags = self.flags.replace(flag, "");
    }
    pub fn set_password(&mut self, password: &str) {
        if let Ok(pwd) = bcrypt::hash(password, bcrypt::DEFAULT_COST) {
            self.password = Some(pwd);
        }
    }
    pub fn verify_password(&self, password: &str) -> bool {
        if let Some(pwd) = &self.password {
            if let Ok(verify) = bcrypt::verify(password, &pwd) {
                return verify;
            }
        }
        false
    }
}
